-- ---------------------------------------------------------------------------------------------------------
-- Official RiftCount Addon
-- Authors: Gabriel Graitzer, Parker Watson
-- Email Address: Riftdevs@gmail.com
-- Version: 1.6
-- Release Date: 9/28
-- Environment: 1.5
--
-- Please contact us at Riftdevs@gmail.com if you encounter any errors or functionality issues or if you
-- wish to suggest features to be included in future updates or if you are a addon author and wish to
-- provide assistance or desire assistance on your own addon.
-- ---------------------------------------------------------------------------------------------------------

dpsMeter = {}
dpsMeter.enabled = true

-- ---------------------------------------------------------------------------------------------------------
-- Create Main UI and set displayed information to default values. Soon to include more appearance frames.
dpsMeter.Context = UI.CreateContext("DPSMeterContext")
dpsMeter.Context:SetPoint("TOPLEFT", UIParent, "TOPLEFT")
dpsMeter.Context:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT")
dpsMeter.Control = UI.CreateFrame("Mask", "Main", dpsMeter.Context)

local RCFrame = dpsMeter.Control
dpsMeter.Background = UI.CreateFrame("Texture", "UI Background", RCFrame)
dpsMeter.Background:SetTexture("RiftCount", "Background.png")
dpsMeter.Footer = UI.CreateFrame("Texture", "UI Background", RCFrame)
dpsMeter.Footer:SetTexture("RiftCount", "Footer.png")

dpsMeter.Chart = UI.CreateFrame("Mask", "Chart Mask", RCFrame)

dpsMeter.Sizer = UI.CreateFrame("Texture", "Sizer", dpsMeter.Context)
dpsMeter.Sizer:SetTexture("RiftCount", "Sizer.png")
dpsMeter.Sizer:SetLayer(31)

dpsMeter.Window = UI.CreateFrame("Text", "Timer", RCFrame)
dpsMeter.PostButton = UI.CreateFrame("Text", "Post Meters Button", RCFrame)
dpsMeter.MenuButton = UI.CreateFrame("Text", "RiftCount Menu Button", RCFrame)
dpsMeter.Post = UI.CreateFrame("RiftWindow", "Meter Posting Info", dpsMeter.Context)
dpsMeter.Post:SetVisible(false)
dpsMeter.PostText = UI.CreateFrame("RiftTextfield", "Top Info", dpsMeter.Post)
dpsMeter.PostClose = UI.CreateFrame("RiftButton", "Close Post Page", dpsMeter.Post)

-- ---------------------------------------------------------------------------------------------------------
-- Default Position Information, will be savedvariable soon
dpsMeter.default = {
	align = "TOPLEFT",
	x = UIParent:GetWidth()*.7,
	y = UIParent:GetHeight()*.3,
	width = dpsMeter.Footer:GetTextureWidth() / 2,
	height = dpsMeter.Background:GetTextureHeight() / 2,
	background_height = dpsMeter.Background:GetTextureHeight() / 2,
	footer_height = dpsMeter.Footer:GetTextureHeight() / 2,
	alpha = 1,
	sort_mode = "dps",
	UI_mode = "rift",
	refresh = 0.25,
	scale = 1,
	locked = false
}

filter = {}

local function savedVars(params)
	if params == "RiftCount" then
		if not dpsMeterSettings then dpsMeterSettings = dpsMeter.default end
		for k,_ in pairs(dpsMeter.default) do
			if not dpsMeterSettings[k] then dpsMeterSettings[k] = dpsMeter.default[k] end
		end
	end

	dpsMeter.settings = dpsMeterSettings

	RCFrame:SetPoint(dpsMeter.settings.align, dpsMeter.Context, dpsMeter.settings.align, dpsMeter.settings.x, dpsMeter.settings.y)
	RCFrame:SetBackgroundColor(0,0,0,0)
	RCFrame:SetLayer(0)

	Background = dpsMeter.Background

	dpsMeter.Window:SetText("00:00")
	dpsMeter.Window:SetHeight(dpsMeter.Window:GetFullHeight())
	dpsMeter.Window:SetWidth(dpsMeter.Window:GetFullWidth())
	dpsMeter.Window:SetLayer(10)
	dpsMeter.Window:SetVisible(false)

	dpsMeter.Background:SetHeight(dpsMeter.settings.background_height)
	dpsMeter.Background:SetWidth(dpsMeter.settings.width)
	dpsMeter.Background:SetPoint("TOPCENTER", RCFrame, "TOPCENTER")
	dpsMeter.Background:SetLayer(1)
	dpsMeter.Background:SetAlpha(dpsMeter.settings.alpha)

	dpsMeter.Footer:SetHeight(dpsMeter.settings.footer_height)
	dpsMeter.Footer:SetWidth(dpsMeter.settings.width)
	dpsMeter.Footer:SetPoint("BOTTOMCENTER", RCFrame, "BOTTOMCENTER")
	dpsMeter.Footer:SetLayer(30)
	dpsMeter.Footer:SetAlpha(dpsMeter.settings.alpha)

	dpsMeter.PostButton:SetText("Post")
	dpsMeter.PostButton:ResizeToText()
	dpsMeter.PostButton:SetBackgroundColor(0,0,0,0.3)
	dpsMeter.PostButton:SetPoint("CENTERLEFT", dpsMeter.Footer, "CENTERLEFT", dpsMeter.settings.width / 8, 0)
	dpsMeter.PostButton:SetLayer(100)

	dpsMeter.Post:SetPoint("CENTER", UIParent, "CENTER")
	dpsMeter.Post:SetVisible(false)

	dpsMeter.PostClose:SetText("Close")
	dpsMeter.PostClose:SetPoint("BOTTOMCENTER", dpsMeter.Post, "BOTTOMCENTER", 0, -(dpsMeter.Post:GetHeight() / 12) / 3)

	top = "No DPS Information Yet"
	dpsMeter.PostText:SetText(top)
	dpsMeter.PostText:SetPoint("CENTER", dpsMeter.Post, "CENTER")
	dpsMeter.PostText:SetWidth(dpsMeter.Post:GetWidth()*.8)
	dpsMeter.PostText:SetHeight(dpsMeter.Post:GetHeight()*.5)
	dpsMeter.PostText:SetSelection(0, 22)

	dpsMeter.MenuButton:SetText("Menu")
	dpsMeter.MenuButton:ResizeToText()
	dpsMeter.MenuButton:SetBackgroundColor(0,0,0,0.3)
	dpsMeter.MenuButton:SetPoint("CENTERRIGHT", dpsMeter.Footer, "CENTERRIGHT", -dpsMeter.settings.width / 8, 0)
	dpsMeter.MenuButton:SetLayer(100)

	dpsMeter.Sizer:SetWidth(dpsMeter.settings.width / 8)
	dpsMeter.Sizer:SetHeight(dpsMeter.Sizer:GetWidth() / 2)
	dpsMeter.Sizer:SetPoint("CENTER", dpsMeter.Footer, "BOTTOMCENTER")
	dpsMeter.Sizer:SetAlpha(dpsMeter.settings.alpha)

	RCFrame:SetHeight(dpsMeter.settings.height)
	RCFrame:SetWidth(dpsMeter.settings.width)

	dpsMeter.Chart:SetPoint("TOPLEFT", RCFrame, "TOPLEFT", (29/512)*(dpsMeter.settings.width*dpsMeterSettings.scale), (192/1024)*(dpsMeter.settings.height*dpsMeterSettings.scale))
	dpsMeter.Chart:SetPoint("BOTTOMRIGHT", dpsMeter.Footer, "TOPRIGHT", -((dpsMeterSettings.scale*dpsMeter.settings.width)*(28/512)), (dpsMeterSettings.scale*dpsMeter.settings.footer_height)*(20/110))
	
	dpsMeter.Chart:SetLayer(40)

	dpsMeter.Window:SetPoint("BOTTOMCENTER", RCFrame, "BOTTOMCENTER", 0, -5)
	dpsMeter.Window:SetBackgroundColor(.3,0,0,1)

	RCUI = {}
	RCUI.Box = dpsMeter.Control
	RCUI.Background = dpsMeter.Background
	RCUI.Footer = dpsMeter.Footer
	RCUI.Sizer = dpsMeter.Sizer
	RCUI.Chart = dpsMeter.Chart
	RCUI.PostButton = dpsMeter.PostButton
	RCUI.Timer = dpsMeter.Window
	RCUI.MenuButton = dpsMeter.MenuButton


	-- ------------------------------------------------------------------------------------------
	-- UI Implementation

	-- ------------------------------------------------------------------------------------------
	-- ---------- Tabs (List on Left, Always up

	RCMenu = {}
	RCMenu.Window = UI.CreateFrame("RiftWindow", "RiftCount Menu Window", dpsMeter.Context)
	RCMenu.Window.Content = RCMenu.Window:GetContent()
	RCMenu.Window:SetTitle("RiftCount Options")
	RCMenu.Window:SetController("content")
	RCMenu.Window:SetVisible(false)
	RCMenu.Window:SetLayer(10)
	RCMenu.List = UI.CreateFrame("Mask", "Settings Mask", RCMenu.Window.Content)
	RCMenu.List:SetBackgroundColor(0,0,0,0.5)
	RCMenu.List:SetPoint("TOPLEFT", RCMenu.Window, "TOPLEFT", 2, 10)
	RCMenu.List:SetPoint("BOTTOM", RCMenu.Window, "BOTTOM", nil, -2)
	RCMenu.List:SetLayer(11)
	RCMenu.Settings = {}
	RCMenu.Settings.AbilityFilter = UI.CreateFrame("Text", "RiftCount", RCMenu.List)
	RCMenu.Settings.AbilityFilter:SetText("Ability Filter")
	RCMenu.Settings.Sort = UI.CreateFrame("Text", "RiftCount", RCMenu.List)
	RCMenu.Settings.Sort:SetText("Sorting")
	RCMenu.Settings.History = UI.CreateFrame("Text", "RiftCount", RCMenu.List)
	RCMenu.Settings.History:SetText("History")
	RCMenu.Settings.UI = UI.CreateFrame("Text", "RiftCount", RCMenu.List)
	RCMenu.Settings.UI:SetText("Interface")
	RCMenu.Close = UI.CreateFrame("RiftButton", "Close Button for Menu", RCMenu.Window)
	RCMenu.Close:SetText("Close")
	RCMenu.Close:SetPoint("BOTTOMRIGHT", RCMenu.Window, "BOTTOMRIGHT", -5, -5)

	function RCMenu.Close.Event:LeftClick()
		RCMenu.Window:SetVisible(false)
		RCMenu.Settings.AbilityFilter.AddText:SetKeyFocus(false)
		RCMenu.Settings.Sort.TimerInput:SetKeyFocus(false)
		RCMenu.Settings.History.Top:SetKeyFocus(false)
	end

	previous = nil

	for k,v in pairs(RCMenu.Settings) do
		if previous then
			v:SetPoint("TOPLEFT", previous, "BOTTOMLEFT")
			v:SetPoint("RIGHT", previous, "RIGHT")
		else
			v:SetPoint("TOPLEFT", RCMenu.List, "TOPLEFT")
			v:SetPoint("RIGHT", RCMenu.List, "RIGHT")
		end
		previous = v

		v.Options = UI.CreateFrame("Frame", "Settings Window", RCMenu.Window.Content)
		v.Options:SetVisible(false)
		v.Options:SetPoint("TOPLEFT", RCMenu.List, "TOPRIGHT", 5, 0)
		v.Options:SetPoint("BOTTOM", RCMenu.Close, "TOP")
		v.Options:SetPoint("RIGHT", RCMenu.Window.Content, "RIGHT", -5, nil)
		v.Options:SetBackgroundColor(0,0,0,0.1)
		v.Options:SetLayer(10)

		v:SetHeight(v:GetFullHeight())
		v:SetLayer(20)
		v.active = false

		function v.Event:MouseIn()
			v:SetBackgroundColor(1,1,1,0.5)
		end
		function v.Event:MouseOut()
			if not v.active then
				v:SetBackgroundColor(0,0,0,0)
			else
				v:SetBackgroundColor(0.5,0,0,0.5)
			end
		end

		function v.Event:LeftClick()
			v:SetBackgroundColor(0.5,0,0,0.5)
			for k,v in pairs(RCMenu.Settings) do
				if v.active then
					v.active = false
					v:SetBackgroundColor(0,0,0,0)
					v.Options:SetVisible(false)
				end
			end
			v.active = true
			v.Options:SetVisible(true)
		end
	end

	previous = nil

	-- ------------------------------------------------------------------------------------------
	-- ----------Ability Filter Settings Page

	RCMenu.Settings.AbilityFilter.List = UI.CreateFrame("Mask", "Ability Filter List", RCMenu.Settings.AbilityFilter.Options)
	RCMenu.Settings.AbilityFilter.List:SetPoint("TOPLEFT", RCMenu.Settings.AbilityFilter.Options, "TOPLEFT")
	RCMenu.Settings.AbilityFilter.List:SetPoint("BOTTOM", RCMenu.Settings.AbilityFilter.Options, "BOTTOM", nil, -40)
	RCMenu.Settings.AbilityFilter.List:SetWidth(100)
	RCMenu.Settings.AbilityFilter.List:SetBackgroundColor(0,0,0,0.5)

	RCMenu.Settings.AbilityFilter.Ability = {}

	RCMenu.Settings.AbilityFilter.Add = UI.CreateFrame("RiftButton", "Add Ability to Filter", RCMenu.Settings.AbilityFilter.Options)
	RCMenu.Settings.AbilityFilter.Add:SetPoint("TOPLEFT", RCMenu.Settings.AbilityFilter.List, "BOTTOMLEFT")
	RCMenu.Settings.AbilityFilter.Add:SetText("Add")
	RCMenu.Settings.AbilityFilter.AddText = UI.CreateFrame("RiftTextfield", "Ability to Add", RCMenu.Settings.AbilityFilter.Options)
	RCMenu.Settings.AbilityFilter.AddText:SetPoint("CENTERLEFT", RCMenu.Settings.AbilityFilter.Add, "CENTERRIGHT")
	RCMenu.Settings.AbilityFilter.AddText:SetBackgroundColor(0,0,0,1)
	RCMenu.Settings.AbilityFilter.AddText:SetLayer(12)
	RCMenu.Settings.AbilityFilter.AddTextBackdrop = UI.CreateFrame("Frame", "Backdrop", RCMenu.Settings.AbilityFilter.Options)
	RCMenu.Settings.AbilityFilter.AddTextBackdrop:SetPoint("TOPLEFT", RCMenu.Settings.AbilityFilter.AddText, "TOPLEFT", -2, -2)
	RCMenu.Settings.AbilityFilter.AddTextBackdrop:SetPoint("BOTTOMRIGHT", RCMenu.Settings.AbilityFilter.AddText, "BOTTOMRIGHT", 2, 2)
	RCMenu.Settings.AbilityFilter.AddTextBackdrop:SetBackgroundColor(0,0,0,0.7)
	RCMenu.Settings.AbilityFilter.AddTextBackdrop:SetLayer(11)
	RCMenu.Settings.AbilityFilter.AddText:SetHeight(18)
	RCMenu.Settings.AbilityFilter.AddText:SetPoint("RIGHT", RCMenu.Settings.AbilityFilter.Options, "RIGHT", -10, nil)

	function RCMenu.Settings.AbilityFilter.Add.Event:LeftClick()
		text = RCMenu.Settings.AbilityFilter.AddText:GetText()
		if text ~= "" then
			addFilter(text)
		end
		updateFilter()
	end

	RCMenu.Settings.AbilityFilter.Remove = UI.CreateFrame("RiftButton", "Remove Ability from Filter", RCMenu.Settings.AbilityFilter.Options)
	RCMenu.Settings.AbilityFilter.Remove:SetText("Remove")
	RCMenu.Settings.AbilityFilter.Remove:SetPoint("CENTERRIGHT", RCMenu.Settings.AbilityFilter.Options, "CENTERRIGHT", -15, -50)

	for k,v in pairs(RCMenu.Settings.AbilityFilter.Ability) do
		function v.Event:MouseIn()
			v:SetBackgroundColor(1,1,1,0.5)
		end
		function v.Event:MouseOut()
			if not v.active then
				v:SetBackgroundColor(0,0,0,0)
			else
				v:SetBackgroundColor(0.5,0,0,0.5)
			end
		end
	end

	function RCMenu.Settings.AbilityFilter.Remove.Event:LeftClick()
		for k,v in pairs(RCMenu.Settings.AbilityFilter.Ability) do
			if v.active then
				for index,value in pairs(filter) do
					if v:GetText() == value then
						addFilter(value)
						v:ClearAll()
						v.active = false
						v:SetBackgroundColor(0,0,0,0)
					end
				end
			end
		end
		updateFilter()
	end

	updateFilter()

	-- ------------------------------------------------------------------------------------------
	-- ----------Sorting Settings Page

	RCMenu.Settings.Sort.ModeTitle = UI.CreateFrame("Text", "Sorting Mode Title", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.ModeTitle:SetText("Sort Mode:")
	RCMenu.Settings.Sort.ModeTitle:ResizeToText()
	RCMenu.Settings.Sort.ModeTitle:SetPoint("TOPLEFT", RCMenu.Settings.Sort.Options, "TOPLEFT", 20, 20)
	RCMenu.Settings.Sort.HPStitle = UI.CreateFrame("Text", "Sorting Mode HPS", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.HPStitle:SetText("HPS:")
	RCMenu.Settings.Sort.HPStitle:ResizeToText()
	RCMenu.Settings.Sort.HPStitle:SetPoint("CENTERLEFT", RCMenu.Settings.Sort.ModeTitle, "CENTERRIGHT", 15, 0)
	RCMenu.Settings.Sort.HPS = UI.CreateFrame("RiftCheckbox", "HPS Checkbox", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.HPS:SetChecked(false)
	RCMenu.Settings.Sort.HPS:SetPoint("CENTERLEFT", RCMenu.Settings.Sort.HPStitle, "CENTERRIGHT", 5, 0)
	RCMenu.Settings.Sort.DPStitle = UI.CreateFrame("Text", "Sorting Mode DPS", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.DPStitle:SetText("DPS")
	RCMenu.Settings.Sort.DPStitle:ResizeToText()
	RCMenu.Settings.Sort.DPStitle:SetPoint("CENTERLEFT", RCMenu.Settings.Sort.HPS, "CENTERRIGHT", 10, 0)
	RCMenu.Settings.Sort.DPS = UI.CreateFrame("RiftCheckbox", "HPS Checkbox", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.DPS:SetChecked(false)
	RCMenu.Settings.Sort.DPS:SetPoint("CENTERLEFT", RCMenu.Settings.Sort.DPStitle, "CENTERRIGHT", 5, 0)

	if dpsMeter.settings.sort_mode == "dps" then
		RCMenu.Settings.Sort.DPS:SetChecked(true)
	else
		RCMenu.Settings.Sort.HPS:SetChecked(true)
	end

	function RCMenu.Settings.Sort.DPS.Event:CheckboxChange()
		if RCMenu.Settings.Sort.DPS:GetChecked() == true then
			dpsMeter.settings.sort_mode = "dps"
			sortmode("dps")
			if RCMenu.Settings.Sort.HPS:GetChecked() == true then
				RCMenu.Settings.Sort.HPS:SetChecked(false)
			end
		else
			dpsMeter.settings.sort_mode = "hps"
			sortmode("hps")
			if RCMenu.Settings.Sort.HPS:GetChecked() == false then
				RCMenu.Settings.Sort.HPS:SetChecked(true)
			end
		end
	end

	function RCMenu.Settings.Sort.HPS.Event:CheckboxChange()
		if RCMenu.Settings.Sort.HPS:GetChecked() == true then
			dpsMeter.settings.sort_mode = "hps"
			if RCMenu.Settings.Sort.DPS:GetChecked() == true then
				RCMenu.Settings.Sort.DPS:SetChecked(false)
			end
		else
			dpsMeter.settings.sort_mode = "dps"
			if RCMenu.Settings.Sort.DPS:GetChecked() == false then
				RCMenu.Settings.Sort.DPS:SetChecked(true)
			end
		end
		--renderChart()
	end

	RCMenu.Settings.Sort.Timer = UI.CreateFrame("Text", "Timer Rate Title", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.Timer:SetText("Refresh Timer:")
	RCMenu.Settings.Sort.Timer:ResizeToText()
	RCMenu.Settings.Sort.Timer:SetPoint("TOPLEFT", RCMenu.Settings.Sort.ModeTitle, "BOTTOMLEFT", 0, 20)
	RCMenu.Settings.Sort.TimerInput = UI.CreateFrame("RiftTextfield", "Timer Rate Inpute", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.TimerInput:SetText(tostring(dpsMeter.settings.refresh))
	RCMenu.Settings.Sort.TimerInput:SetWidth(60)
	RCMenu.Settings.Sort.TimerInput:SetBackgroundColor(0,0,0,0.9)
	RCMenu.Settings.Sort.TimerInput:SetPoint("CENTERLEFT", RCMenu.Settings.Sort.Timer, "CENTERRIGHT", 5, 0)
	RCMenu.Settings.Sort.TimerSet = UI.CreateFrame("RiftButton", "Timer Set", RCMenu.Settings.Sort.Options)
	RCMenu.Settings.Sort.TimerSet:SetText("Set")
	RCMenu.Settings.Sort.TimerSet:SetPoint("CENTERLEFT", RCMenu.Settings.Sort.TimerInput, "CENTERRIGHT")

	function RCMenu.Settings.Sort.TimerSet.Event:LeftPress()
		if tonumber(RCMenu.Settings.Sort.TimerInput:GetText()) then
			dpsMeter.settings.refresh = tonumber(RCMenu.Settings.Sort.TimerInput:GetText())
			RCMenu.Settings.Sort.TimerInput:SetKeyFocus(false)
		end
	end

	-- ------------------------------------------------------------------------------------------
	-- ----------History Settings Page

	RCMenu.Settings.History.Title = UI.CreateFrame("Text", "History Title", RCMenu.Settings.History.Options)
	RCMenu.Settings.History.Title:SetText("Target:")
	RCMenu.Settings.History.Title:ResizeToText()
	RCMenu.Settings.History.Title:SetPoint("TOPLEFT", RCMenu.Settings.History.Options, "TOPLEFT", 5, 0)
	RCMenu.Settings.History.List = UI.CreateFrame("Mask", "Ability Filter List", RCMenu.Settings.History.Options)
	RCMenu.Settings.History.List:SetPoint("TOP", RCMenu.Settings.History.Title, "BOTTOM")
	RCMenu.Settings.History.List:SetPoint("BOTTOMLEFT", RCMenu.Settings.History.Options, "BOTTOMLEFT")
	RCMenu.Settings.History.List:SetWidth(100)
	RCMenu.Settings.History.List:SetBackgroundColor(0,0,0,0.5)

	RCMenu.Settings.History.Frame = {}
	RCMenu.Settings.History.Frame[1] = UI.CreateFrame("Text", "History Frames", RCMenu.Settings.History.List)
	RCMenu.Settings.History.Frame[1]:SetPoint("TOPLEFT", RCMenu.Settings.History.List, "TOPLEFT")

	function RCMenu.Settings.History.List.Event:WheelForward()
		if RCMenu.Settings.History.Frame[1]:GetTop() - RCMenu.Settings.History.List:GetTop() > 0 then
			RCMenu.Settings.History.Frame[1]:SetPoint("TOPLEFT", RCMenu.Settings.History.List, "TOPLEFT", 0, (RCMenu.Settings.History.Frame[1]:GetTop() - RCMenu.Settings.History.List:GetTop()) - 2)
		end
	end
	
	function RCMenu.Settings.History.List.Event:WheelBack()
		RCMenu.Settings.History.Frame[1]:SetPoint("TOPLEFT", RCMenu.Settings.History.List, "TOPLEFT", 0, (RCMenu.Settings.History.Frame[1]:GetTop() - RCMenu.Settings.History.List:GetTop()) + 2)
	end

	for k,v in pairs(RCMenu.Settings.History.Frame) do
		v.available = false
		function v.Event:MouseIn()
			if v.available then
				v:SetBackgroundColor(1,1,1,0.5)
			end
		end
		function v.Event:MouseOut()
			if v.available then
				if not v.active then
					v:SetBackgroundColor(0,0,0,0)
				else
					v:SetBackgroundColor(0.5,0,0,0.5)
				end
			end
		end
		function v.Event:LeftClick()
			if v.available then
				for k,v in pairs(RCMenu.Settings.History.Frame) do
					v.active = false
					v:SetBackgroundColor(0,0,0,0)
				end
				v.active = true
				v:SetBackgroundColor(0.5,0,0,0.5)
				updateHistory()
			end
		end
	end

	RCMenu.Settings.History.Chart = {}
	RCMenu.Settings.History.ChartFrame = {}

	RCMenu.Settings.History.Chart = UI.CreateFrame("Mask", "History Chart", RCMenu.Settings.History.Options)
	RCMenu.Settings.History.Chart:SetPoint("TOPLEFT", RCMenu.Settings.History.List, "TOPRIGHT", 10, 0)
	RCMenu.Settings.History.Chart:SetPoint("BOTTOMRIGHT", RCMenu.Settings.History.Options, "BOTTOMRIGHT", -10, -40)
	RCMenu.Settings.History.Chart:SetBackgroundColor(0,0,0,.3)

	previous = nil

	for i=1,20 do
		RCMenu.Settings.History.ChartFrame[i] = UI.CreateFrame("Text", "History Chart Frame", RCMenu.Settings.History.Chart)
		if i == 1 then
			RCMenu.Settings.History.ChartFrame[i]:SetPoint("TOPLEFT", RCMenu.Settings.History.Chart, "TOPLEFT")
			RCMenu.Settings.History.ChartFrame[i]:SetPoint("TOPRIGHT", RCMenu.Settings.History.Chart, "TOPRIGHT")
			previous = RCMenu.Settings.History.ChartFrame[i]
		else
			RCMenu.Settings.History.ChartFrame[i]:SetPoint("TOPLEFT", previous, "BOTTOMLEFT")
			RCMenu.Settings.History.ChartFrame[i]:SetPoint("TOPRIGHT", previous, "BOTTOMRIGHT")
			previous = RCMenu.Settings.History.ChartFrame[i]
		end
	end
	
	RCMenu.Settings.History.Top = UI.CreateFrame("RiftTextfield", "History Top for Posting", RCMenu.Settings.History.Options)
	RCMenu.Settings.History.Top:SetPoint("BOTTOMLEFT", RCMenu.Settings.History.List, "BOTTOMRIGHT", 5, 0)
	RCMenu.Settings.History.Top:SetBackgroundColor(0,0,0,0.2)
	RCMenu.Settings.History.Top:SetPoint("RIGHT", RCMenu.Settings.History.Options, "RIGHT", -10, nil)
	
	function RCMenu.Settings.History.Top.Event:LeftClick()
		if hEncounter then
			if hEncounter.top then
				RCMenu.Settings.History.Top:SetCursor(0)
				RCMenu.Settings.History.Top:SetSelection(0, string.len(hEncounter.top))
			end
		end
	end

	-- ------------------------------------------------------------------------------------------
	-- ----------UI Settings Page

	RCMenu.Settings.UI.Alpha = UI.CreateFrame("Text", "RiftCount", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.Alpha:SetText("Opacity")
	RCMenu.Settings.UI.Alpha:SetPoint("TOPLEFT", RCMenu.Settings.UI.Options, "TOPLEFT", 5, 10)
	RCMenu.Settings.UI.Alpha:ResizeToText()
	RCMenu.Settings.UI.AlphaSet = UI.CreateFrame("RiftTextfield", "RiftCount", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.AlphaSet:SetPoint("CENTERLEFT", RCMenu.Settings.UI.Alpha, "CENTERRIGHT")
	RCMenu.Settings.UI.AlphaSet:SetBackgroundColor(0,0,0,1)
	RCMenu.Settings.UI.AlphaSet:SetText(tostring(dpsMeter.settings.alpha))
	RCMenu.Settings.UI.AlphaSet:SetWidth(80)
	RCMenu.Settings.UI.AlphaApply = UI.CreateFrame("RiftButton", "RiftCount", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.AlphaApply:SetText("Set")
	RCMenu.Settings.UI.AlphaApply:SetPoint("CENTERLEFT", RCMenu.Settings.UI.AlphaSet, "CENTERRIGHT", 5, 0)

	function RCMenu.Settings.UI.AlphaApply.Event:LeftPress()
		if tonumber(RCMenu.Settings.UI.AlphaSet:GetText()) then
			dpsMeterSettings.alpha = tonumber(RCMenu.Settings.UI.AlphaSet:GetText())
			dpsMeter.Background:SetAlpha(dpsMeterSettings.alpha)
			dpsMeter.Footer:SetAlpha(dpsMeterSettings.alpha)
			dpsMeter.Sizer:SetAlpha(dpsMeterSettings.alpha)
		end
	end

	RCMenu.Settings.UI.LockTitle = UI.CreateFrame("Text", "Lock Settings Title", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.LockTitle:SetPoint("TOPLEFT", RCMenu.Settings.UI.Alpha, "BOTTOMLEFT", 0, 10)
	RCMenu.Settings.UI.LockTitle:SetText("Locked:")
	RCMenu.Settings.UI.LockTitle:ResizeToText()
	RCMenu.Settings.UI.Lock = UI.CreateFrame("RiftCheckbox", "Lock Toggle", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.Lock:SetPoint("CENTERLEFT", RCMenu.Settings.UI.LockTitle, "CENTERRIGHT", 5, 0)

	if dpsMeterSettings.locked then
		RCMenu.Settings.UI.Lock:SetChecked(true)
	else
		RCMenu.Settings.UI.Lock:SetChecked(false)
	end

	function RCMenu.Settings.UI.Lock.Event:CheckboxChange()
		if RCMenu.Settings.UI.Lock:GetChecked() then
			dpsMeterSettings.locked = true
		else
			dpsMeterSettings.locked = false
		end
	end

	RCMenu.Settings.UI.Scale = UI.CreateFrame("Text", "Scale Title", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.Scale:SetPoint("TOPLEFT", RCMenu.Settings.UI.LockTitle, "BOTTOMLEFT", 0, 10)
	RCMenu.Settings.UI.Scale:SetText("Scale:")
	RCMenu.Settings.UI.Scale:ResizeToText()

	RCMenu.Settings.UI.ScaleSlider = UI.CreateFrame("RiftSlider", "Scale Slider", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.ScaleSlider:SetPoint("CENTERLEFT", RCMenu.Settings.UI.Scale, "CENTERRIGHT", 5, 0)
	RCMenu.Settings.UI.ScaleSlider:SetWidth(200)
	RCMenu.Settings.UI.ScaleSlider:SetRange(0, 200)
	RCMenu.Settings.UI.ScaleSlider:SetPosition(math.floor(dpsMeterSettings.scale * 100))

	function RCMenu.Settings.UI.ScaleSlider.Event:SliderChange()
		dpsMeterSettings.scale = (RCMenu.Settings.UI.ScaleSlider:GetPosition() / 100)
		updateScale()
	end

	RCMenu.Settings.UI.ScaleValue = UI.CreateFrame("Text", "Scale Value", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.ScaleValue:SetPoint("CENTERLEFT", RCMenu.Settings.UI.ScaleSlider, "CENTERRIGHT", 10, 0)
	RCMenu.Settings.UI.ScaleValue:SetText(tostring(dpsMeterSettings.scale))

	Defaults = {}
	
	defaultsDone = false
	
	function updateScale()
		for k,v in pairs(RCUI) do
			--print(k)
			if not Defaults[k] then Defaults[k] = {} end
			if not defaultsDone then Defaults[k].width = v:GetWidth() end
			if not defaultsDone then Defaults[k].height = v:GetHeight() end
			if k == "PostButton" and not defaultsDone then Defaults[k].fontSize = v:GetFontSize() end
			if k == "MenuButton" and not defaultsDone then Defaults[k].fontSize = v:GetFontSize() end
			v:SetHeight(Defaults[k].height*dpsMeterSettings.scale)
			v:SetWidth(Defaults[k].width*dpsMeterSettings.scale)
			if Defaults[k].fontSize then 
				v:SetFontSize(Defaults[k].fontSize*dpsMeterSettings.scale)
			end
		end
		
		-- if dpsMeter.PlayerDMG then 
			-- dpsMeter.PlayerDMG:SetPoint("TOPLEFT", Background, "TOPLEFT", (29/512)*dpsMeter.settings.width*dpsMeterSettings.scale, (95/1024)*dpsMeter.settings.background_height*dpsMeterSettings.scale)
			-- dpsMeter.PlayerDMG:SetPoint("TOPRIGHT", Background, "TOPRIGHT", -(29/512)*dpsMeter.settings.width*dpsMeterSettings.scale, (95/1024)*dpsMeter.settings.background_height*dpsMeterSettings.scale)
		-- end
		
		if dpsMeter.PlayerDMG then render() end
		
		RCMenu.Settings.UI.ScaleValue:SetText(tostring(dpsMeterSettings.scale))
	end

	updateScale()
	
	defaultsDone = true

	RCMenu.Settings.UI.Themes = {}
	RCMenu.Settings.UI.Themes.Title = UI.CreateFrame("Text", "Themes title", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.Themes.Title:SetText("Theme:")
	RCMenu.Settings.UI.Themes.Title:SetPoint("TOPLEFT", RCMenu.Settings.UI.Scale, "BOTTOMLEFT", 0, 10)
	RCMenu.Settings.UI.Themes.List = UI.CreateFrame("Mask", "Themes List", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.Themes.List:SetPoint("TOPLEFT", RCMenu.Settings.UI.Themes.Title, "TOPRIGHT", 5, 0)
	RCMenu.Settings.UI.Themes.List:SetWidth(100)
	RCMenu.Settings.UI.Themes.List:SetBackgroundColor(0,0,0,0.7)
	RCMenu.Settings.UI.Themes.List:SetLayer(15)
	RCMenu.Settings.UI.Themes.ListItem = {}
	RCMenu.Settings.UI.Themes.ListItem.Rift = UI.CreateFrame("Text", "Rift Theme Option", RCMenu.Settings.UI.Themes.List)
	RCMenu.Settings.UI.Themes.ListItem.Rift:SetText("Rift")
	RCMenu.Settings.UI.Themes.ListItem.Rift:ResizeToText()
	RCMenu.Settings.UI.Themes.ListItem.Min = UI.CreateFrame("Text", "Minimalist Theme Option", RCMenu.Settings.UI.Themes.List)
	RCMenu.Settings.UI.Themes.ListItem.Min:SetText("Minimalist")
	RCMenu.Settings.UI.Themes.ListItem.Min:ResizeToText()

	previous = nil
	
	for k,v in pairs(RCMenu.Settings.UI.Themes.ListItem) do
		if not previous then
			v:SetPoint("TOPLEFT", RCMenu.Settings.UI.Themes.List, "TOPLEFT")
			v:SetLayer(16)
			previous = v
		else
			v:SetPoint("TOPLEFT", previous, "BOTTOMLEFT")
			v:SetLayer(16)
			previous = v
		end
		
		v.active = false
		
		function v.Event:MouseIn()
			v:SetBackgroundColor(1,1,1,0.5)
		end
		function v.Event:MouseOut()
			if not v.active then
				v:SetBackgroundColor(0,0,0,0)
			else
				v:SetBackgroundColor(0.5,0,0,0.5)
			end
		end
		function v.Event:LeftClick()
			for k,v in pairs(RCMenu.Settings.UI.Themes.ListItem) do
				if v.active then v.active = false end
				v:SetBackgroundColor(0,0,0,0)
			end
			
			v.active = true
			v:SetBackgroundColor(0.5,0,0,0.5)
			
			if v == RCMenu.Settings.UI.Themes.ListItem.Rift then
				UI_Rift()
			else
				UI_Min()
			end
		end
	end
	
	RCMenu.Settings.UI.Reset = UI.CreateFrame("RiftButton", "Reset Button", RCMenu.Settings.UI.Options)
	RCMenu.Settings.UI.Reset:SetText("Reset")
	RCMenu.Settings.UI.Reset:SetPoint("BOTTOMLEFT", RCMenu.Settings.UI.Options, "BOTTOMLEFT", 0, -10)

	function RCMenu.Settings.UI.Reset.Event:LeftPress()
		savedVarsReset()
	end
	
	if dpsMeter.settings.UI_mode == "min" then
		Background:SetVisible(false)
		dpsMeter.Footer:SetVisible(false)
		RCFrame:SetBackgroundColor(0,0,0,dpsMeterSettings.alpha)
	end
end

local function savedVarsUpdate(params)
	if params == Inspect.Addon.Current() then
		if dpsMeter.settings then
			for k,_ in pairs(dpsMeter.settings) do
				if dpsMeter.settings[k] then dpsMeterSettings[k] = dpsMeter.settings[k] end
			end
		end
	end
end

function savedVarsReset()
	for k,_ in pairs(dpsMeter.default) do
		dpsMeterSettings[k] = dpsMeter.default[k]
		dpsMeter.settings[k] = dpsMeter.default[k]
	end
	
	for k,v in pairs(RCUI) do
		if Defaults[k].fontSize then v:SetFontSize(Defaults[k].fontSize) end
	end
	RCFrame:SetPoint(dpsMeter.settings.align, dpsMeter.Context, dpsMeter.settings.align, dpsMeter.settings.x, dpsMeter.settings.y)
	RCFrame:SetAlpha(dpsMeterSettings.alpha)
	dpsMeter.Background:SetAlpha(dpsMeterSettings.alpha)
	dpsMeter.Footer:SetAlpha(dpsMeterSettings.alpha)
	dpsMeter.Sizer:SetAlpha(dpsMeterSettings.alpha)
	updateScale()
	UI_Rift()
	print("Default Settings Restored.")
end

local function center()
	RCFrame:SetPoint("CENTER", UIParent, "CENTER")
	print("UI Centered")
end

function dpsMeter.PostButton.Event:LeftClick()
	dpsMeter.Post:SetVisible(true)
end

function dpsMeter.MenuButton.Event:LeftClick()
	menu()
end

function dpsMeter.PostText.Event:LeftDown()
	self.MouseDown = true
end
function dpsMeter.PostText.Event:LeftUp()
	if self.MouseDown == true then
		self.MouseDown = false
		dpsMeter.PostText:SetCursor(0)
		dpsMeter.PostText:SetSelection(0,string.len(top))
	end
end

function dpsMeter.PostClose.Event:LeftDown()
	self.MouseDown = true
end
function dpsMeter.PostClose.Event:LeftUp()
	if self.MouseDown == true then
		self.MouseDown = false
		dpsMeter.Post:SetVisible(false)
		dpsMeter.PostText:SetKeyFocus(false)
	end
end

-- ---------------------------------------------------------------------------------------------------------
-- Create Frame Pet and Player tables for frame creation and damage tracking.
dpsMeter.Raid = {}

-- ---------------------------------------------------------------------------------------------------------
-- Player table created to display Self seperate from player amongst other encounter.group members.
pos = 0
history = {}
encounter = {}
fight = {}
encounter.player = {}
encounter.group = {}
encounter.groupSorted = {}

-- ---------------------------------------------------------------------------------------------------------
-- Sets values for firstRun and init ("Initialized") to false when addon is loaded.
local firstRun = true
local init = false

-- ---------------------------------------------------------------------------------------------------------
-- Function triggered by Event.System.Secure.Enter, triggers when player enters combat. This begins the
-- timer and handles all the work of creating the frames for each encounter.group or raid member as well as the "Self"
-- frame. Gathers Unit ID information for each encounter.group member and pet for indexing in the damageRecord function.
-- Resets all players damage and dps counters to 0 as well as resetting all frames from previous recordings.
--
local function startTimer()
	if dpsMeter.enabled then
		time = Inspect.Time.Frame()
		refreshTimerReset = Inspect.Time.Frame()
		init = true
		encounter.player.dmg = 0
		encounter.player.heal = 0
		reset()
	end
end

--local function preLoad()
--	init = true
--	time = Inspect.Time.Frame()
--	refreshTimerReset = Inspect.Time.Frame()
--	player.dmg = 0
--	player.heal = 0
--end

-- ---------------------------------------------------------------------------------------------------------
-- Function triggered by Event.System.Update.Begin, updates once per frame render, used to track dps and
-- update the dps charts in real time.
--
local function update()
	if dpsMeter.enabled == true then
	if init and dpsMeter.enabled then
		tempTime = Inspect.Time.Frame() - time
		refreshTimer = Inspect.Time.Frame() - refreshTimerReset

		dpsMeter.Window:SetText(string.format("%d:%02d", math.floor(tempTime / 60), tempTime % 60))
		dpsMeter.Window:SetHeight(dpsMeter.Window:GetFullHeight())
		dpsMeter.Window:SetWidth(dpsMeter.Window:GetFullWidth())

		encounter.totaldps = encounter.totalheal / tempTime
		encounter.totalhps = encounter.totalhps / tempTime

		encounter.player.dps = encounter.player.dmg / tempTime
		encounter.player.hps = encounter.player.heal / tempTime

		for k, v in pairs(encounter.group) do
			if v.dmg >= 0 then
				v.dps = v.dmg / tempTime
			end
			if v.heal >= 0 then
				v.hps = v.heal / tempTime
			end
		end

		if dpsMeter.settings.sort_mode == "dps" then table.sort(encounter.groupSorted, function(a, b) return encounter.group[a].dmg > encounter.group[b].dmg end)
		elseif dpsMeter.settings.sort_mode == "hps" then table.sort(encounter.groupSorted, function(a, b) return encounter.group[a].heal > encounter.group[b].heal end) end

		if refreshTimer > dpsMeter.settings.refresh and pos == 0 then
			dpsMeter.Playertext:SetText(encounter.player.name.." dps: "..string.format("%d", encounter.player.dps).." hps: "..string.format("%d", encounter.player.hps))
			dpsMeter.Playertext:ResizeToText()
			for k, v in pairs(encounter.groupSorted) do
				detail = Inspect.Unit.Detail(v)
				if detail.calling == "cleric" then encounter.group[v].color = {0,0.55,0.1,0.3}
				elseif detail.calling == "rogue" then 	encounter.group[v].color = {0.8,0.7,0,0.3}
				elseif detail.calling == "mage" then encounter.group[v].color = {0.5,0,0.5,0.3}
				elseif detail.calling == "warrior" then encounter.group[v].color = {0.5,0,0,0.3}
				else encounter.group[v].color = {0,0,0,0.3} end
				dpsMeter.Raid[k]:SetBackgroundColor(unpack(encounter.group[v].color))
				dpsMeter.Raid[k]:SetText(string.sub(encounter.group[v].name, 1, 12).." dps: "..string.format("%d", encounter.group[v].dps).." hps: "..string.format("%d", encounter.group[v].hps))
				dpsMeter.Raid[k]:SetHeight(dpsMeter.Raid[k]:GetFullHeight())
			end
			refreshTimerReset = Inspect.Time.Frame()
		end
	end
	end
end

-- ---------------------------------------------------------------------------------------------------------
-- Function triggered by Event.System.Secure.Leave, ends the combat tracking timer displaying an accurate
-- timer for length of time in combat, also sets "initialized" to false to stop recording damage.
--
-- !API Update 1.6!
-- Add Tracking for Individual Combat Timers
--
local function endTimer()
	if dpsMeter.enabled == true then
		finalTime = Inspect.Time.Frame() - time
		init = false
		dpsMeter.Window:SetText(string.format("%02d:%02d", finalTime / 60 % 60, finalTime % 60))
		encounter.totaldps = encounter.totaldmg / finalTime
		encounter.finalTime = finalTime
		updateTop()
		encounter.top = top
		fight = {}
		fight = deepcopy(encounter)
		if encounter.target then table.insert(history, 1, fight) end
		updateHistoryList()
	end
end

local function playerEntered_group()
	encounter.groupSorted = {}
	encounter.group = {}
	encounter.pet = {}
end

local function playerLeft_group()
	encounter.groupSorted = {}
	encounter.group = {}
	encounter.pet = {}
	for i=1,20 do
		dpsMeter.Raid[i]:SetVisible(false)
	end
end

local function groupAdd(unitID, unitSpecifier)
	detail = Inspect.Unit.Detail(unitID)
	encounter.group[unitID] = {name = detail.name, calling = detail.calling, dmg = 0, dps = 0, heal = 0, hps = 0, color = {0,0,0,0}}
	encounter.groupSorted[#encounter.groupSorted + 1] = unitID
	dpsMeter.Raid[#encounter.groupSorted]:SetVisible(true)
	if detail.calling == "cleric" then encounter.group[unitID].color = {0,0.55,0.1,0.3}
	elseif detail.calling == "rogue" then 	encounter.group[unitID].color = {0.8,0.7,0,0.3}
	elseif detail.calling == "mage" then encounter.group[unitID].color = {0.5,0,0.5,0.3}
	elseif detail.calling == "warrior" then encounter.group[unitID].color = {0.5,0,0,0.3}
	else encounter.group[unitID].color = {0,0,0,0.3} end
	dpsMeter.Raid[#encounter.groupSorted]:SetBackgroundColor(unpack(encounter.group[unitID].color))
	dpsMeter.Raid[#encounter.groupSorted]:SetText(string.sub(encounter.group[unitID].name, 1, 12).." dps: "..encounter.group[unitID].dps.." hps: "..encounter.group[unitID].hps)
end

local function groupLeave(unitID, unitSpecifier)
	encounter.group[unitID] = nil
	if #encounter.groupSorted > 1 then
		local counter = 1
		dpsMeter.Raid[#encounter.groupSorted]:SetVisible(false)
		encounter.groupSorted = {}
		
		for i=1,20 do
			unitSpecifier = string.format("group%02d", i)
			detail = Inspect.Unit.Detail(unitSpecifier)
			if detail then
				encounter.groupSorted[counter] = Inspect.Unit.Lookup(unitSpecifier)
				counter = counter + 1
			end
		end

		sortmode(dpsMeter.settings.sort_mode)

	else
		dpsMeter.Raid[1]:SetVisible(false)
		encounter.groupSorted = {}
	end
end

local function PetAdd(unitID, ownerID)
	detail = Inspect.Unit.Detail(unitID)
	encounter.pet[unitID] = {name = detail.name, owner = ownerID, dmg = 0, dps = 0, heal = 0, hps = 0}
end

local function PetLeave(unitID, ownerID)
	encounter.pet[unitID] = nil
end

local function PlayerPetAdd(unitID, ownerID)
	encounter.player.pet = unitID
end

local function PlayerPetRemove()
	encounter.player.pet = nil
end

function RCFrame.Event:LeftDown()
	if not dpsMeter.settings.locked then
		self.MouseDown = true
		mouseData = Inspect.Mouse()
		self.MyStartX = RCFrame:GetLeft()
		self.MyStartY = RCFrame:GetTop()
		self.StartX = mouseData.x - self.MyStartX
		self.StartY = mouseData.y - self.MyStartY
		tempX = RCFrame:GetLeft()
		tempY = RCFrame:GetTop()
		tempW = RCFrame:GetWidth()
		tempH =	RCFrame:GetHeight()
		RCFrame:ClearAll()
		RCFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", tempX, tempY)
		dpsMeter.Chart:SetPoint("TOPLEFT", RCFrame, "TOPLEFT", (29/512)*(dpsMeter.settings.width*dpsMeterSettings.scale), (192/1024)*(dpsMeter.settings.background_height*dpsMeterSettings.scale))
		dpsMeter.Chart:SetPoint("BOTTOMRIGHT", dpsMeter.Footer, "TOPRIGHT", -((dpsMeterSettings.scale*dpsMeter.settings.width)*(28/512)), (dpsMeterSettings.scale*dpsMeter.settings.footer_height)*(20/110))
		RCFrame:SetWidth(tempW)
		RCFrame:SetHeight(tempH)
	end
end
function RCFrame.Event:MouseMove()
	if self.MouseDown then
		local newX, newY
		mouseData = Inspect.Mouse()
		newX = mouseData.x - self.StartX
		newY = mouseData.y - self.StartY
		RCFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", newX, newY)
		dpsMeter.Chart:SetPoint("TOPLEFT", RCFrame, "TOPLEFT", (29/512)*(dpsMeter.settings.width*dpsMeterSettings.scale), (192/1024)*(dpsMeter.settings.background_height*dpsMeterSettings.scale))
		dpsMeter.Chart:SetPoint("BOTTOMRIGHT", dpsMeter.Footer, "TOPRIGHT", -((dpsMeterSettings.scale*dpsMeter.settings.width)*(28/512)), (dpsMeterSettings.scale*dpsMeter.settings.footer_height)*(20/110))
	end
end
function RCFrame.Event:LeftUp()
	if self.MouseDown then
		dpsMeter.settings.x = math.floor(RCFrame:GetLeft())
		dpsMeter.settings.y = math.floor(RCFrame:GetTop())
		self.MouseDown = false
		dpsMeterSettings.x = dpsMeter.settings.x
		dpsMeterSettings.y = dpsMeter.settings.y
		dpsMeter.Chart:SetPoint("TOPLEFT", RCFrame, "TOPLEFT", (29/512)*(dpsMeter.settings.width*dpsMeterSettings.scale), (192/1024)*(dpsMeter.settings.background_height*dpsMeterSettings.scale))
		dpsMeter.Chart:SetPoint("BOTTOMRIGHT", dpsMeter.Footer, "TOPRIGHT", -((dpsMeterSettings.scale*dpsMeter.settings.width)*(28/512)), (dpsMeterSettings.scale*dpsMeter.settings.footer_height)*(20/110))
	end
end

function dpsMeter.Sizer.Event:LeftDown()
	self.MouseDown = true
	mouseData = Inspect.Mouse()
	self.MyStartX = RCFrame:GetLeft()
	self.MyStartY = RCFrame:GetBottom()
	self.StartX = mouseData.x - self.MyStartX
	self.StartY = mouseData.y - self.MyStartY
	tempX = RCFrame:GetLeft()
	tempY = RCFrame:GetBottom()
	tempW = RCFrame:GetWidth()
	tempH =	RCFrame:GetHeight()
	RCFrame:SetHeight(tempH)
	self:SetBackgroundColor(0,0,0,0.5)
end
function dpsMeter.Sizer.Event:MouseMove()
	if self.MouseDown then
		local newX, newY
		mouseData = Inspect.Mouse()
		newH = mouseData.y - RCFrame:GetTop()
		if (newH <= (194/1024)*(dpsMeter.settings.background_height*dpsMeterSettings.scale)) or (newH >= (dpsMeter.settings.background_height*dpsMeterSettings.scale)*(530/512)) then
			self.MouseDown = false
		else
			RCFrame:SetHeight(newH)
			dpsMeter.settings.height = newH
		end
	end
end
function dpsMeter.Sizer.Event:LeftUp()
	if self.MouseDown then
		dpsMeter.settings.height = RCFrame:GetHeight()
		self.MouseDown = false
		dpsMeterSettings.height = dpsMeter.settings.height
	end
	self:SetBackgroundColor(0, 0, 0, 0)
end

-- ---------------------------------------------------------------------------------------------------------
-- Function triggered by Event.Combat.Damage, tracks damage done by each player and adds it to their total
-- damage done. Tracks pet damage as well.
local function damageRecord(params)
	if dpsMeter.enabled == true and params then
		if init and params.damage then
			detail = Inspect.Unit.Detail(params.caster)
			damage = params.damage
			unitID = params.caster

			if LibSRM.Group.UnitExists(unitID) then
				if encounter.group[unitID] then
					encounter.group[unitID].dmg = encounter.group[unitID].dmg + damage
					encounter.totaldmg = encounter.totaldmg + damage
					if Inspect.Unit.Detail(params.target) then
						encounter.target = Inspect.Unit.Detail(params.target).name
					end
				end
			elseif LibSRM.Group.PetExists(unitID) then
				unitID, ownerID = LibSRM.Group.PetExists(unitID)
				if encounter.pet[unitID] then
					if encounter.group[ownerID] then
						encounter.group[ownerID].dmg = encounter.group[ownerID].dmg + damage
						encounter.totaldmg = encounter.totaldmg + damage
					end
				end
			end
			if unitID == encounter.player.id or unitID == encounter.player.pet then
				encounter.player.dmg = encounter.player.dmg + damage
				if not encounter.target then
					encounter.target = Inspect.Unit.Detail(unitID).name
				end
			end
			
			if not encounter.target then
				encounter.target = Inspect.Unit.Detail(unitID).name
			end
		end
	end
end

local function healRecord(params)
	if dpsMeter.enabled == true and params then
		if init and params.heal then
			detail = Inspect.Unit.Detail(params.caster)
			heal = params.heal
			unitID = params.caster

			if LibSRM.Group.UnitExists(unitID) then
				if encounter.group[unitID] then
					encounter.group[unitID].heal = encounter.group[unitID].heal + heal
					encounter.totalheal = encounter.totalheal + heal
				end
			elseif LibSRM.Group.PetExists(unitID) then
				unitID, ownerID = LibSRM.Group.PetExists(unitID)
				if encounter.pet[unitID] then
					if encounter.group[ownerID] then
						encounter.group[ownerID].heal = encounter.group[ownerID].heal + heal
						encounter.totalheal = encounter.totalheal + heal
					end
				end
			end
			if unitID == encounter.player.id or unitID == encounter.player.pet then
				encounter.player.heal = encounter.player.heal + heal
			end
		end
	end
end

-- ---------------------------------------------------------------------------------------------------------
-- Initial funtion, launches when addon is done loading, used to provide initial information.
local function loadSettings(addonname)
	if addonname == "RiftCount" then
		print("RiftCount Loaded Successfully")
		print("For more info please type /RCHelp")
	end
end

-- ---------------------------------------------------------------------------------------------------------
-- Menu UI Window

function updateHistory()
	for i=1,20 do
		RCMenu.Settings.History.ChartFrame[i]:SetVisible(false)
	end
	for k,v in pairs(RCMenu.Settings.History.Frame) do
		if v.active then
			hEncounter = history[k]
			hGroup = history[k].group
			hGroupSorted = history[k].groupSorted
			
			RCMenu.Settings.History.Top:SetText(hEncounter.top)
			
			for i=1,20 do
				RCMenu.Settings.History.ChartFrame[i]:SetBackgroundColor(0,0,0,0)
				RCMenu.Settings.History.ChartFrame[i]:SetVisible(false)
			end
			
			if dpsMeter.settings.sort_mode == "dps" then table.sort(history[k].groupSorted, function(a, b) return history[k].group[a].dmg > history[k].group[b].dmg end)
			elseif dpsMeter.settings.sort_mode == "hps" then table.sort(history[k].groupSorted, function(a, b) return history[k].group[a].heal > history[k].group[b].heal end) end
			
			for key,value in pairs(RCMenu.Settings.History.ChartFrame) do
				if hGroupSorted[key] then
					unit = hGroup[hGroupSorted[key]]
					value:SetText(string.sub(unit.name,1,8).." DPS: "..string.format("%0d",unit.dps).." HPS: "..string.format("%0d",unit.hps))
					value:SetHeight(value:GetFullHeight())
					value:SetBackgroundColor(unpack(hGroup[hGroupSorted[key]].color))
					value:SetVisible(true)
				end
			end
			do return end
		end
	end
end

function updateFilter()
	if filter then
		table.sort(filter)
		for k,v in pairs(filter) do
			local new = false

			if RCMenu.Settings.AbilityFilter.Ability[v] then
				RCMenu.Settings.AbilityFilter.Ability[v]:ClearAll()
			else
				RCMenu.Settings.AbilityFilter.Ability[v] = UI.CreateFrame("Text", "Ability Filter Item", RCMenu.Settings.AbilityFilter.List)
				RCMenu.Settings.AbilityFilter.Ability[v].new = true
			end

			if previous then
				RCMenu.Settings.AbilityFilter.Ability[v]:SetText(v)
				RCMenu.Settings.AbilityFilter.Ability[v]:SetHeight(RCMenu.Settings.AbilityFilter.Ability[v]:GetFullHeight())
				RCMenu.Settings.AbilityFilter.Ability[v]:SetPoint("TOPLEFT", previous, "BOTTOMLEFT")
				RCMenu.Settings.AbilityFilter.Ability[v]:SetPoint("TOPRIGHT", previous, "BOTTOMRIGHT")
				previous = RCMenu.Settings.AbilityFilter.Ability[v]
			else
				RCMenu.Settings.AbilityFilter.Ability[v]:SetText(v)
				RCMenu.Settings.AbilityFilter.Ability[v]:SetPoint("TOPLEFT", RCMenu.Settings.AbilityFilter.List, "TOPLEFT")
				RCMenu.Settings.AbilityFilter.Ability[v]:SetPoint("TOPRIGHT", RCMenu.Settings.AbilityFilter.List, "TOPRIGHT")
				RCMenu.Settings.AbilityFilter.Ability[v]:SetHeight(RCMenu.Settings.AbilityFilter.Ability[v]:GetFullHeight())
				previous = RCMenu.Settings.AbilityFilter.Ability[v]
			end

			RCMenu.Settings.AbilityFilter.Ability[v].active = false
		end
	end

	for k,v in pairs(RCMenu.Settings.AbilityFilter.Ability) do
		if v.new then
			function v.Event:LeftClick()
				if not v.active then
					v:SetBackgroundColor(0.5,0,0,0.5)
					v.active = true
				else
					v:SetBackgroundColor(0,0,0,0)
					v.active = false
				end
			end

			function v.Event:MouseIn()
				v:SetBackgroundColor(1,1,1,0.5)
			end
			function v.Event:MouseOut()
				if not v.active then
					v:SetBackgroundColor(0,0,0,0)
				else
					v:SetBackgroundColor(0.5,0,0,0.5)
				end
			end
			v.new = false
		end
		v.active = false
	end
	previous = nil
end

function updateHistoryList()
	for i=1,#history do
		if not RCMenu.Settings.History.Frame[i] then 
			RCMenu.Settings.History.Frame[i] = UI.CreateFrame("Text", "History Frame", RCMenu.Settings.History.List) 
			RCMenu.Settings.History.Frame[i]:SetPoint("TOPLEFT", RCMenu.Settings.History.Frame[i-1], "BOTTOMLEFT")
		end
		RCMenu.Settings.History.Frame[i]:SetText(history[i].target)
		RCMenu.Settings.History.Frame[i]:ResizeToText()
		RCMenu.Settings.History.Frame[i].available = true
		RCMenu.Settings.History.Frame[i].active = false
		RCMenu.Settings.History.Frame[i]:SetVisible(true)
	end
	for k,v in pairs(RCMenu.Settings.History.Frame) do
		function v.Event:MouseIn()
				if v.available then
					v:SetBackgroundColor(1,1,1,0.5)
				end
			end
			function v.Event:MouseOut()
				if v.available then
					if not v.active then
						v:SetBackgroundColor(0,0,0,0)
					else
						v:SetBackgroundColor(0.5,0,0,0.5)
					end
				end
			end
			function v.Event:LeftClick()
				if v.available then
					for k,v in pairs(RCMenu.Settings.History.Frame) do
						v.active = false
						v:SetBackgroundColor(0,0,0,0)
					end
					v.active = true
					v:SetBackgroundColor(0.5,0,0,0.5)
					updateHistory()
				end
			end
	end
	updateHistory()
end

function menu()
	RCMenu.Window:SetPoint("CENTER", UIParent, "CENTER")
	RCMenu.Window:SetVisible(true)
	RCMenu.List:SetWidth(100)
end

function AddBorder(parent, width, r, g, b, a)
  local  frame = {}
  frame.left = UI.CreateFrame("Frame","Border", parent)
  frame.left:SetWidth(width)
  frame.left:SetPoint("TOPRIGHT", parent, "TOPLEFT")
  frame.left:SetPoint("BOTTOMRIGHT", parent, "BOTTOMLEFT")
  frame.left:SetBackgroundColor(r, g, b, a)

  frame.top = UI.CreateFrame("Frame","Border", parent)
  frame.top:SetHeight(width)
  frame.top:SetPoint("BOTTOMLEFT", parent, "TOPLEFT", 0 - width, 0)
  frame.top:SetPoint("BOTTOMRIGHT", parent, "TOPRIGHT", width, 0)
  frame.top:SetBackgroundColor(r, g, b, a)

  frame.right = UI.CreateFrame("Frame","Border", parent)
  frame.right:SetWidth(width)
  frame.right:SetPoint("TOPLEFT", parent, "TOPRIGHT")
  frame.right:SetPoint("BOTTOMLEFT", parent, "BOTTOMRIGHT")
  frame.right:SetBackgroundColor(r, g, b, a)

  frame.bottom = UI.CreateFrame("Frame","Border", parent)
  frame.bottom:SetHeight(width)
  frame.bottom:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", 0 - width, 0)
  frame.bottom:SetPoint("TOPRIGHT", parent, "BOTTOMRIGHT", width, 0)
  frame.bottom:SetBackgroundColor(r, g, b, a)
  return frame
end

function sortmode(mode)
	if string.lower(mode) == "dps" then dpsMeter.settings.sort_mode = "dps"
	elseif string.lower(mode) == "hps" then dpsMeter.settings.sort_mode = "hps"
	elseif string.lower(mode) == "none" then dpsMeter.settings.sort_mode = "none"
	else print("RCSort Usage: /RCSort mode where mode = dps, hps or none")
	end

	if pos ~= 0 then

		if dpsMeter.settings.sort_mode == "dps" then table.sort(history[pos].groupSorted, function(a, b) return history[pos].group[a].dmg > history[pos].group[b].dmg end)
		elseif dpsMeter.settings.sort_mode == "hps" then table.sort(history[pos].groupSorted, function(a, b) return history[pos].group[a].heal > history[pos].group[b].heal end) end

		for k, v in pairs(history[pos].groupSorted) do
			dpsMeter.Raid[k]:SetBackgroundColor(unpack(history[pos].group[v].color))
			dpsMeter.Raid[k]:SetText(string.sub(history[pos].group[v].name, 1, 12).." dps: "..string.format("%."..(0).."f", tostring(history[pos].group[v].dps)).." hps: "..string.format("%."..(0).."f", tostring(history[pos].group[v].hps)))
			dpsMeter.Raid[k]:SetHeight(dpsMeter.Raid[k]:GetFullHeight())
		end
	else

		if dpsMeter.settings.sort_mode == "dps" then table.sort(encounter.groupSorted, function(a, b) return encounter.group[a].dmg > encounter.group[b].dmg end)
		elseif dpsMeter.settings.sort_mode == "hps" then table.sort(encounter.groupSorted, function(a, b) return encounter.group[a].heal > encounter.group[b].heal end) end

		for k, v in pairs(encounter.groupSorted) do
			dpsMeter.Raid[k]:SetBackgroundColor(unpack(encounter.group[v].color))
			dpsMeter.Raid[k]:SetText(string.sub(encounter.group[v].name, 1, 12).." dps: "..string.format("%."..(0).."f", tostring(encounter.group[v].dps)).." hps: "..string.format("%."..(0).."f", tostring(encounter.group[v].hps)))
			dpsMeter.Raid[k]:SetHeight(dpsMeter.Raid[k]:GetFullHeight())
		end
	end
end

local function setEnabled()
	if dpsMeter.enabled == true then
		dpsMeter.enabled = false
	else
		dpsMeter.enabled = true
	end
	if dpsMeter.enabled == false then
		init = false
		RCFrame:SetVisible(false)
	elseif dpsMeter.enabled == true then
		RCFrame:SetVisible(true)
		dpsMeter.Sizer:SetVisible(false)
	end
end

function UI_Min()
	dpsMeter.Background:SetVisible(false)
	dpsMeter.Footer:SetVisible(false)
	RCFrame:SetBackgroundColor(0,0,0,0.5)
	dpsMeter.settings.UI_mode = "min"
end

function UI_Rift()
	dpsMeter.Background:SetVisible(true)
	dpsMeter.Footer:SetVisible(true)
	dpsMeter.settings.UI_mode = "rift"
	RCFrame:SetBackgroundColor(0,0,0,0)
end

function reset()
	for k,v in pairs(encounter.groupSorted) do
		encounter.group[v].dmg = 0
		encounter.group[v].dps = 0
		encounter.group[v].heal = 0
		encounter.group[v].hps = 0
	end

	encounter.totalheal = 0
	encounter.totalhps = 0
	encounter.totaldmg = 0
	encounter.totaldps = 0
	encounter.player.dmg = 0
	encounter.player.dps = 0
	encounter.player.heal = 0
	encounter.player.hps = 0
	time = Inspect.Time.Frame()
	refreshTimerReset = Inspect.Time.Frame()
end

local function setRefresh(var)
	dpsMeter.settings.refresh = tonumber(var)
end

local function setHeight(var)
	dpsMeter.settings.height = (dpsMeter.settings.height)*tonumber(var)
	dpsMeter.settings.background_height = (dpsMeter.settings.background_height)*tonumber(var)
	dpsMeter.settings.footer_height = (dpsMeter.settings.footer_height)*tonumber(var)
	print("Height Scaled by ", var, "x")
	render()
end

local function setWidth(var)
	dpsMeter.settings.width = (dpsMeter.settings.width)*tonumber(var)
	print("Width Scaled by ", var, "x")
	render()
end

local function setAlpha(var)
	if dpsMeter.settings.UI_mode == "min" then RCFrame:SetAlpha(tonumber(var)) end
	dpsMeter.settings.alpha = tonumber(var)
	render()
end

local function setLock()
	if dpsMeter.settings.locked then dpsMeter.settings.locked = false
	else dpsMeter.settings.locked = true end
end

helper = {}
function helper.resolveName(name)
    local a = _G
    for key in string.gmatch(name, "([^%.]+)(%.?)") do
        print(key)
        if a[key] then
            a = a[key]
        else
            return nil
        end
    end
    return a
end

function helper.functionExists(name)
    return type(helper.resolveName(name)) == 'function'
end

function addFilter(var)
	for k, v in pairs(filter) do
		if v == var then
			RCMenu.Settings.AbilityFilter.Ability[v]:ClearAll()
			RCMenu.Settings.AbilityFilter.Ability[v].active = false
			table.remove(filter, k)
			do return end
		end
	end
	table.insert(filter, var)
	for k,v in pairs(RCMenu.Settings.AbilityFilter.Ability) do
		if not v.Event then
			function v.Event:MouseIn()
				v:SetBackgroundColor(1,1,1,0.5)
			end
			function v.Event:MouseOut()
				if not v.active then
					v:SetBackgroundColor(0,0,0,0)
				else
					v:SetBackgroundColor(0.5,0,0,0.5)
				end
			end
		end
	end
	updateFilter()
end

local function historyCurrent()
	pos = 0
	renderChart()
end

local function rcpos()
	if pos == 0 then
		print("You are at position 0, therefore you are viewing the current fight")
	else
		print(pos.."out of "..#history)
	end
end

local function historyBack()
	if history[pos+1] then
		pos = pos + 1
	else
		print("You have reached the end of the history log.")
	end
	if pos == 1 then
		print("You are now viewing position 1, this is the most recent fight that has ended")
	end
	renderChart()
end

local function historyForward()
	if pos > 0 then
		pos = pos - 1
	else
		print("You are at the current dps charts, cannot go further forward")
	end
	renderChart()
end

function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

function updateTop()
	if pos ~= 0 then
		if #history[pos].groupSorted > 0 and dpsMeter.settings.sort_mode == "dps" then
			topList = {}
			top = string.format("%2d", history[pos].finalTime).."s -:DPS:- Total: "..string.format("%."..(0).."f", tostring(history[pos].totaldps)).." "
			for i = 1,#history[pos].groupSorted do
				topList[i] = " - "..i..". "..history[pos].group[history[pos].groupSorted[i]].name..": "..string.format("%."..(0).."f", tostring(history[pos].group[history[pos].groupSorted[i]].dps))
				top = top..topList[i]
			end
			dpsMeter.PostText:SetText(top)
			dpsMeter.PostText:SetSelection(1,string.len(top))
		elseif #history[pos].groupSorted > 0 and dpsMeter.settings.sort_mode == "hps" then
			topList = {}
			history[pos].totalhps = history[pos].totalheal / finalTime
			top = string.format("%2d", history[pos].finalTime).."s -:HPS:- Total: "..string.format("%."..(1).."f", tostring(history[pos].totalhps)).." "
			for i = 1,#history[pos].groupSorted do
				topList[i] = " - "..i..". "..history[pos].group[history[pos].groupSorted[i]].name..": "..string.format("%."..(0).."f", tostring(history[pos].group[history[pos].groupSorted[i]].hps))
				top = top..topList[i]
			end
			dpsMeter.PostText:SetText(top)
			dpsMeter.PostText:SetSelection(1,string.len(top))
		else
			top = string.format("%2d", finalTime).."s "..history[pos].player.name.." DPS: "..string.format("%."..(0).."f", tostring(history[pos].player.dps)).." HPS: "..string.format("%."..(0).."f", tostring(history[pos].player.hps))
		end
	else
		if encounter.finalTime then
			if #encounter.groupSorted > 0 and dpsMeter.settings.sort_mode == "dps" then
				topList = {}
				top = string.format("%2d", encounter.finalTime).."s -:DPS:- Total: "..string.format("%."..(0).."f", tostring(encounter.totaldps)).." "
				for i = 1,#encounter.groupSorted do
					topList[i] = " - "..i..". "..encounter.group[encounter.groupSorted[i]].name..": "..string.format("%."..(0).."f", tostring(encounter.group[encounter.groupSorted[i]].dps))
					top = top..topList[i]
				end
				dpsMeter.PostText:SetText(top)
				dpsMeter.PostText:SetSelection(1,string.len(top))
			elseif #encounter.groupSorted > 0 and dpsMeter.settings.sort_mode == "hps" then
				topList = {}
				encounter.totalhps = encounter.totalheal / finalTime
				top = string.format("%2d", encounter.finalTime).."s -:HPS:- Total: "..string.format("%."..(1).."f", tostring(encounter.totalhps)).." "
				for i = 1,#encounter.groupSorted do
					topList[i] = " - "..i..". "..encounter.group[encounter.groupSorted[i]].name..": "..string.format("%."..(0).."f", tostring(encounter.group[encounter.groupSorted[i]].hps))
					top = top..topList[i]
				end
				dpsMeter.PostText:SetText(top)
				dpsMeter.PostText:SetSelection(1,string.len(top))
			else
				top = string.format("%2d", encounter.finalTime).."s "..encounter.player.name.." DPS: "..string.format("%."..(0).."f", tostring(encounter.player.dps)).." HPS: "..string.format("%."..(0).."f", tostring(encounter.player.hps))
				dpsMeter.PostText:SetText(top)
				dpsMeter.PostText:SetSelection(1,string.len(top))
			end
		end
	end
	
	for k,v in pairs(RCMenu.Settings.History.Frame) do
		if v.active then
			local encounter = history[group]
			local top = encounter.top
			
			if #encounter.groupSorted > 0 and dpsMeter.settings.sort_mode == "dps" then
				topList = {}
				top = string.format("%2d", encounter.finalTime).."s -:DPS:- Total: "..string.format("%."..(0).."f", tostring(encounter.totaldps)).." "
				for i = 1,#encounter.groupSorted do
					topList[i] = " - "..i..". "..encounter.group[encounter.groupSorted[i]].name..": "..string.format("%."..(0).."f", tostring(encounter.group[encounter.groupSorted[i]].dps))
					top = top..topList[i]
				end
				RCMenu.Settings.History.Top:SetText(top)
				RCMenu.Settings.History.Top:SetSelection(1,string.len(top))
			elseif #encounter.groupSorted > 0 and dpsMeter.settings.sort_mode == "hps" then
				topList = {}
				encounter.totalhps = encounter.totalheal / finalTime
				top = string.format("%2d", encounter.finalTime).."s -:HPS:- Total: "..string.format("%."..(1).."f", tostring(encounter.totalhps)).." "
				for i = 1,#encounter.groupSorted do
					topList[i] = " - "..i..". "..encounter.group[encounter.groupSorted[i]].name..": "..string.format("%."..(0).."f", tostring(encounter.group[encounter.groupSorted[i]].hps))
					top = top..topList[i]
				end
				RCMenu.Settings.History.Top:SetText(top)
				RCMenu.Settings.History.Top:SetSelection(1,string.len(top))
			else
				top = string.format("%2d", encounter.finalTime).."s "..encounter.player.name.." DPS: "..string.format("%."..(0).."f", tostring(encounter.player.dps)).." HPS: "..string.format("%."..(0).."f", tostring(encounter.player.hps))
				RCMenu.Settings.History.Top:SetText(top)
				RCMenu.Settings.History.Top:SetSelection(1,string.len(top))
			end
		end
	end
end

function renderChart()
	if pos ~= 0 then

		print("History Fight: "..pos.."      Target: "..history[pos].target)

		dpsMeter.Playertext:SetText(history[pos].player.name.." dps: "..string.format("%d", history[pos].player.dps).." hps: "..string.format("%d", history[pos].player.hps))
		dpsMeter.Playertext:ResizeToText()

		if #history[pos].groupSorted > 0 then
			if dpsMeter.settings.sort_mode == "dps" then table.sort(history[pos].groupSorted, function(a, b) return history[pos].group[a].dmg > history[pos].group[b].dmg end)
			elseif dpsMeter.settings.sort_mode == "hps" then table.sort(history[pos].groupSorted, function(a, b) return history[pos].group[a].heal > history[pos].group[b].heal end) end

			for k, v in pairs(history[pos].groupSorted) do
				dpsMeter.Raid[k]:SetBackgroundColor(unpack(history[pos].group[v].color))
				dpsMeter.Raid[k]:SetText(string.sub(history[pos].group[v].name, 1, 12).." dps: "..string.format("%."..(0).."f", tostring(history[pos].group[v].dps)).." hps: "..string.format("%."..(0).."f", tostring(history[pos].group[v].hps)))
				dpsMeter.Raid[k]:SetHeight(dpsMeter.Raid[k]:GetFullHeight())
--				dpsMeter.Footer:SetPoint("TOPCENTER", dpsMeter.Raid[k], "BOTTOMCENTER")
			end
		end
	else

		dpsMeter.Playertext:SetText(encounter.player.name.." dps: "..string.format("%d", encounter.player.dps).." hps: "..string.format("%d", encounter.player.hps))
		dpsMeter.Playertext:ResizeToText()

		if #encounter.groupSorted > 0 then
			if dpsMeter.settings.sort_mode == "dps" then table.sort(encounter.groupSorted, function(a, b) return encounter.group[a].dmg > encounter.group[b].dmg end)
			elseif dpsMeter.settings.sort_mode == "hps" then table.sort(encounter.groupSorted, function(a, b) return encounter.group[a].heal > encounter.group[b].heal end) end

			for k, v in pairs(encounter.groupSorted) do
				dpsMeter.Raid[k]:SetBackgroundColor(unpack(encounter.group[v].color))
				dpsMeter.Raid[k]:SetText(string.sub(encounter.group[v].name, 1, 12).." dps: "..string.format("%."..(0).."f", tostring(encounter.group[v].dps)).." hps: "..string.format("%."..(0).."f", tostring(encounter.group[v].hps)))
				dpsMeter.Raid[k]:SetHeight(dpsMeter.Raid[k]:GetFullHeight())
--				dpsMeter.Footer:SetPoint("TOPCENTER", dpsMeter.Raid[k], "BOTTOMCENTER")
			end
		end
	end

	updateTop()
end

function render()
	RCFrame:SetWidth(dpsMeter.settings.width*dpsMeterSettings.scale)
	RCFrame:SetHeight(dpsMeter.settings.height*dpsMeterSettings.scale)
	--if dpsMeter.settings.UI_mode == "min" then
		RCFrame:SetAlpha(dpsMeter.settings.alpha)
	--else
		--RCFrame:SetAlpha(1)
	--end

	dpsMeter.Background:SetWidth(dpsMeter.settings.width*dpsMeterSettings.scale)
	dpsMeter.Background:SetHeight(dpsMeter.settings.background_height*dpsMeterSettings.scale)
	dpsMeter.Background:SetAlpha(dpsMeter.settings.alpha)

	dpsMeter.Footer:SetWidth(dpsMeter.settings.width*dpsMeterSettings.scale)
	dpsMeter.Footer:SetHeight(dpsMeter.settings.footer_height*dpsMeterSettings.scale)
	dpsMeter.Footer:SetAlpha(dpsMeter.settings.alpha)

	dpsMeter.Sizer:SetWidth(dpsMeterSettings.scale*(dpsMeter.settings.width / 6))
	dpsMeter.Sizer:SetHeight(dpsMeterSettings.scale*(dpsMeter.Sizer:GetWidth() / 3))
	dpsMeter.Sizer:SetAlpha(dpsMeter.settings.alpha)

	dpsMeter.PlayerDMG:SetHeight((dpsMeter.settings.background_height-(dpsMeter.settings.background_height*(95/1024)+dpsMeter.settings.background_height*(1024-153)/1024))*dpsMeterSettings.scale)
	--dpsMeter.PlayerDMG:SetWidth((dpsMeter.settings.width-(dpsMeter.settings.width*(28/512)*2))*dpsMeterSettings.scale)
	--dpsMeter.PlayerDMG:SetPoint("TOPLEFT", Background, "TOPLEFT", (29/512)*dpsMeter.settings.width*dpsMeterSettings.scale, (95/1024)*dpsMeter.settings.background_height*dpsMeterSettings.scale)
	dpsMeter.PlayerDMG:SetPoint("TOPLEFT", Background, "TOPLEFT", (29/512)*dpsMeter.settings.width*dpsMeterSettings.scale, (95/1024)*dpsMeter.settings.background_height*dpsMeterSettings.scale)
	dpsMeter.PlayerDMG:SetPoint("TOPRIGHT", Background, "TOPRIGHT", -(29/512)*dpsMeter.settings.width*dpsMeterSettings.scale, (95/1024)*dpsMeter.settings.background_height*dpsMeterSettings.scale)
	
	dpsMeter.Playertext:SetPoint("CENTERLEFT", dpsMeter.PlayerDMG, "CENTERLEFT")

	--dpsMeter.Title:SetPoint("TOPCENTER", RCFrame, "TOPCENTER", 0, (33/1024)*dpsMeter.settings.background_height)
	--dpsMeter.Title:SetHeight(dpsMeter.Title:GetFullHeight())
	--dpsMeter.Title:SetWidth(dpsMeter.Title:GetFullWidth())

	dpsMeter.Window:SetHeight(dpsMeter.Window:GetFullHeight())
	dpsMeter.Window:SetWidth(dpsMeter.Window:GetFullWidth())

	dpsMeter.PostButton:SetPoint("CENTERLEFT", dpsMeter.Footer, "CENTERLEFT", dpsMeter.settings.width / 8, 0)

	dpsMeter.Chart:ClearAll()
	dpsMeter.Chart:SetPoint("TOPLEFT", RCFrame, "TOPLEFT", (29/512)*(dpsMeter.settings.width*dpsMeterSettings.scale), (192/1024)*(dpsMeter.settings.background_height*dpsMeterSettings.scale))
	dpsMeter.Chart:SetPoint("BOTTOMRIGHT", dpsMeter.Footer, "TOPRIGHT", -((dpsMeterSettings.scale*dpsMeter.settings.width)*(28/512)), (dpsMeterSettings.scale*dpsMeter.settings.footer_height)*(20/110))
	
	dpsMeter.PostText:SetPoint("CENTER", dpsMeter.Post, "CENTER")
	dpsMeter.PostText:SetWidth(dpsMeter.Post:GetWidth()*.8)
	dpsMeter.PostText:SetHeight(dpsMeter.Post:GetHeight()*.5)

	--dpsMeter.Raid[1]:SetPoint("TOPLEFT", RCFrame, "TOPLEFT", (29/512)*dpsMeter.settings.width, (194/1024)*dpsMeter.settings.background_height)
	dpsMeter.Raid[1]:ClearAll()
	dpsMeter.Raid[1]:SetPoint("TOPLEFT", dpsMeter.Chart, "TOPLEFT")
	dpsMeter.Raid[1]:SetPoint("TOPRIGHT", dpsMeter.Chart, "TOPRIGHT")
	dpsMeter.Raid[1]:SetHeight(dpsMeter.Raid[1]:GetFullHeight())
	--dpsMeter.Raid[1]:SetWidth(dpsMeter.settings.width-(dpsMeter.settings.width*(28/512)*2))
end
-- ---------------------------------------------------------------------------------------------------------
-- Help Function, provides information to user when requested by /RCHelp
local function displayHelp(command)
	if command == "" then
		print("Welcome to the RiftCount Help Section. Here is a list of the available commands to you:")
		print('/RCHelp -- This help menu (Optional /RCHelp "Command Name" for detailed info *Not Yet Available*)')
		--print('/RCMode "Mode" -- Sets sorting mode (Available Modes: dps, hps and none)')
		print('/RCDefault Resets position to default values, also found in menu as Reset button in UI section')
		print('/RCHide -- Hides the RiftCount window, same command to reenable.')
		print('/RCMenu -- Displays Menu, available also by clicking Menu on bottom of RiftCount Window')
		--print('/RCMinimalist -- Switch to minimalist Rift UI')
		--print('/RCRiftUI -- Switch to Rift Themed UI')
		print('/RCReset -- Resets values to 0')
		--print('/RCRefresh "number (seconds)" -- Changes the refresh timer on when it updates the charts')
		--print('/RCHeight "Scale Value" -- Changes the height of the frame by the amount given (Scaling value, 0.5 will cut height in half, 2 will double height etc)')
		--print('/RCWidth "Scale Value" -- Changes the width of the frame by the amount given (Scaling value, 0.5 will cut width in half, 2 will double width etc)')
		--print('/RCOpac "0-1 Value" -- Sets the Opacity (Transparency or Alpha) of the background, 0 is invisible, 0.5 is half opacity, 1 is solid, etc.')
		--print('/RCLock -- Locks/Unlocks the RiftCount window')
		--print('/RCFilter "Ability Name" -- Add an ability to the filter list, type without ability name to print the current list of abilities to be filtered')
		--print('/RCCurrent -- Return to viewing Current Fight (Last fight if not in combat, same as position 1 in history)')
		--print('/RCBack -- Go back one fight in the history')
		--print('/RCForward -- Go forward one fight in the history')
		--print('/RCPos -- View Current Position in History')
	else
		print('/RCHelp Command Detail Usage: type /RCHelp followed by a slash command name (available commands listed by /RCHelp).')
		print('Example: /RCHelp RCMode')
	end
end

local function playerReady()
	if firstRun == true then
		dpsMeter.PlayerDMG = UI.CreateFrame("Text", "Player DPS", RCFrame)
		dpsMeter.Playertext = UI.CreateFrame("Text", "Player Text", RCFrame)
		firstRun = false
	end

	detail = Inspect.Unit.Detail("player")
	if detail.calling == "cleric" then dpsMeter.PlayerDMG:SetBackgroundColor(0,0.55,0.1,0.3)
	elseif detail.calling == "rogue" then 	dpsMeter.PlayerDMG:SetBackgroundColor(0.8,0.7,0,0.3)
	elseif detail.calling == "mage" then dpsMeter.PlayerDMG:SetBackgroundColor(0.5,0,0.5,0.3)
	elseif detail.calling == "warrior" then dpsMeter.PlayerDMG:SetBackgroundColor(0.5,0,0,0.3)
	else dpsMeter.PlayerDMG:SetBackgroundColor(0,0,0,0.3) end

	dpsMeter.PlayerDMG:SetHeight((dpsMeter.settings.background_height-(dpsMeter.settings.background_height*(95/1024)+dpsMeter.settings.background_height*(1024-153)/1024))*dpsMeterSettings.scale)
	--dpsMeter.PlayerDMG:SetWidth(dpsMeter.settings.width-(dpsMeter.settings.width*(28/512)*2))
	dpsMeter.PlayerDMG:SetPoint("TOPLEFT", Background, "TOPLEFT", (29/512)*dpsMeter.settings.width*dpsMeterSettings.scale, (95/1024)*dpsMeter.settings.background_height*dpsMeterSettings.scale)
	dpsMeter.PlayerDMG:SetPoint("TOPRIGHT", Background, "TOPRIGHT", -(29/512)*dpsMeter.settings.width*dpsMeterSettings.scale, (95/1024)*dpsMeter.settings.background_height*dpsMeterSettings.scale)
	dpsMeter.Playertext:SetPoint("CENTERLEFT", dpsMeter.PlayerDMG, "CENTERLEFT")
	dpsMeter.PlayerDMG:SetLayer(10)
	dpsMeter.Playertext:SetLayer(11)
	
	--RCUI.PlayerDMG = dpsMeter.PlayerDMG
	--RCUI.Playertext = dpsMeter.Playertext

	encounter.totaldps = 0
	encounter.totalhps = 0
	encounter.totaldmg = 0
	encounter.totalheal = 0

	encounter.player = {}
	encounter.player.dmg = 0
	encounter.player.dps = 0
	encounter.player.hps = 0
	encounter.player.heal = 0
	encounter.player.name = detail.name
	encounter.player.id = Inspect.Unit.Lookup("player")
	encounter.player.pet = Inspect.Unit.Lookup("player.pet")

	dpsMeter.Playertext:SetText(detail.name.." dps: "..encounter.player.dps.." hps: "..encounter.player.hps)
	dpsMeter.Playertext:ResizeToText()

	encounter.groupSorted = {}
	encounter.group = {}
	encounter.pet = {}

	dpsMeter.Raid[1] = UI.CreateFrame("Text", "Raid/Party Members", dpsMeter.Chart)
	dpsMeter.Raid[1]:SetPoint("TOPLEFT", RCFrame, "TOPLEFT", (29/512)*dpsMeter.settings.width, (194/1024)*dpsMeter.settings.background_height)
	dpsMeter.Raid[1]:SetText("group Position "..1)
	dpsMeter.Raid[1]:SetHeight(dpsMeter.Raid[1]:GetFullHeight())
	dpsMeter.Raid[1]:SetWidth(dpsMeter.settings.width-(dpsMeter.settings.width*(28/512)*2))
	dpsMeter.Raid[1]:SetVisible(false)

	for i=1,20 do
		if i >= 2 then
			dpsMeter.Raid[i] = UI.CreateFrame("Text", "Raid/Party Members", dpsMeter.Chart)
			--dpsMeter.Raid[i]:SetPoint("TOPCENTER", dpsMeter.Raid[i-1], "BOTTOMCENTER")
			dpsMeter.Raid[i]:SetPoint("TOPLEFT", dpsMeter.Raid[i-1], "BOTTOMLEFT")
			dpsMeter.Raid[i]:SetPoint("TOPRIGHT", dpsMeter.Raid[i-1], "BOTTOMRIGHT")
			dpsMeter.Raid[i]:SetText("encounter.group Position "..i)
			dpsMeter.Raid[i]:SetHeight(dpsMeter.Raid[i]:GetFullHeight())
			--dpsMeter.Raid[i]:SetWidth(dpsMeter.settings.width-(dpsMeter.settings.width*(28/512)*2))
			dpsMeter.Raid[i]:SetLayer(40)
			dpsMeter.Raid[i]:SetVisible(false)
		end
	end
	
	render()
	
	--RCUI.Raid = dpsMeter.Raid

	table.insert (Event.SafesRaidManager.Player.Join, {playerEntered_group, "RiftCount", "Player Entering encounter.group"})
	table.insert (Event.SafesRaidManager.Player.Leave, {playerLeft_group, "RiftCount", "Player Left encounter.group"})
	table.insert (Event.SafesRaidManager.Group.Join, {groupAdd, "RiftCount", "encounter.group Member Added"})
	table.insert (Event.SafesRaidManager.Group.Leave, {groupLeave, "RiftCount", "encounter.group Member Left"})
	table.insert (Event.SafesRaidManager.Pet.Add, {PetAdd, "RiftCount", "encounter.group Pet Added"})
	table.insert (Event.SafesRaidManager.Pet.Remove, {PetLeave, "RiftCount", "encounter.group Pet Removed"})
	table.insert (Event.SafesRaidManager.Player.Pet.Add, {PlayerPetAdd, "RiftCount", "Player Pet Added"})
	table.insert (Event.SafesRaidManager.Player.Pet.Remove, {PlayerPetRemove, "RiftCount", "Player Pet Removed"})
		local frameRefresh = {update, "RiftCount", "Update Timer"}
	table.insert (Event.System.Update.Begin, frameRefresh)
		local combatLeave = {endTimer, "RiftCount", "Combat End"}
	table.insert (Event.System.Secure.Leave, combatLeave)
	table.insert (Event.Combat.Damage, {damageRecord, "RiftCount", "Parse Damage"})
	table.insert (Event.Combat.Heal, {healRecord, "RiftCount", "Parse Heals"})
end
-- ---------------------------------------------------------------------------------------------------------
-- Slash Commands
--
-- !Future Update Information!
-- Read displayHelp function for list of commands that are either available or in planning.
--
table.insert (Command.Slash.Register("RCHelp"), {displayHelp, "RiftCount", "Display Help Text"})
table.insert (Command.Slash.Register("RCMode"), {sortmode, "RiftCount", "Set Sort Mode"})
table.insert (Command.Slash.Register("RCHide"), {setEnabled, "RiftCount", "Set to Enabled or Disabled"})
table.insert (Command.Slash.Register("RCDefault"), {savedVarsReset, "RiftCount", "Reset Saved Variables to Default Values"})
table.insert (Command.Slash.Register("RCCenter"), {center, "RiftCount", "Centers the UI"})
table.insert (Command.Slash.Register("RCMinimalist"), {UI_Min, "RiftCount", "Change UI To Minimalist"})
table.insert (Command.Slash.Register("RCRiftUI"), {UI_Rift, "RiftCount", "Change UI To Rift Theme"})
table.insert (Command.Slash.Register("RCReset"), {reset, "RiftCount", "Reset Damage"})
table.insert (Command.Slash.Register("RCRefresh"), {setRefresh, "RiftCount", "Change Refresh Timer"})
table.insert (Command.Slash.Register("RCHeight"), {setHeight, "RiftCount", "Change Height"})
table.insert (Command.Slash.Register("RCWidth"), {setWidth, "RiftCount", "Change Width"})
table.insert (Command.Slash.Register("RCOpac"), {setAlpha, "RiftCount", "Change Opacity"})
table.insert (Command.Slash.Register("RCLock"), {setLock, "RiftCount", "Changes Locked Status"})
table.insert (Command.Slash.Register("RCCurrent"), {historyCurrent, "RiftCount", "Go to current fight chart"})
table.insert (Command.Slash.Register("RCForward"), {historyForward, "RiftCount", "Go forward one fight in chart history"})
table.insert (Command.Slash.Register("RCBack"), {historyBack, "RiftCount", "Go back one fight in chart history"})
table.insert (Command.Slash.Register("RCPos"), {rcpos, "RiftCount", "View Current Position"})
table.insert (Command.Slash.Register("RCMenu"), {menu, "RiftCount", "RiftCount Settings/Basic Menu"})
table.insert (Command.Slash.Register("RCFilter"), {addFilter, "RiftCount", "RiftCount Ability Filter"})
-- ---------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------
-- Event Triggers
table.insert (Event.Addon.Load.End, {loadSettings, "RiftCount", "Load Settings (Launched on Addon.Load.End)"})
table.insert (Event.Addon.SavedVariables.Load.End, {savedVars, "RiftCount", "Load Saved Variables"})
table.insert (Event.Addon.SavedVariables.Save.Begin, {savedVarsUpdate, "RiftCount", "Updating Saved Variables"})
table.insert (Event.SafesRaidManager.Player.Ready, {playerReady, "RiftCount", "Player Available"})
table.insert (Event.System.Secure.Enter, {startTimer, "RiftCount", "Combat Enter"})
-- ---------------------------------------------------------------------------------------------------------
